package xyz.twovb.sgm.levels;

/*
 * Created by 2vb - 4/6/2024
 */

import org.apache.commons.io.FileUtils;
import org.bukkit.*;
import org.bukkit.block.Block;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.yaml.snakeyaml.DumperOptions;
import org.yaml.snakeyaml.Yaml;
import xyz.twovb.sgm.SGM;
import xyz.twovb.toolbox.api.CustomPlayer;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class LevelManager {

    public static final String levelPath = SGM.getInstance().getDataFolder().getPath() + "/levels/";

    public static final String gamePath = SGM.getInstance().getDataFolder().getPath() + "/games/";

    public static final String mapPath = SGM.getInstance().getDataFolder().getPath() + "/maps/";

    // todo: make it the actual map and not levels
    public HashMap<String, List<String>> enabledMaps = new HashMap<>();

    private static void loadWorld(String worldName, LevelType type) {
        // Check if the world is already loaded
        String key = type.toString().toLowerCase() + "_" + worldName;
        WorldCreator wc = null;
        if (Bukkit.getWorld(new NamespacedKey(SGM.getInstance(), key)) != null) {
            SGM.getInstance().getCLogger().log("World " + worldName + " is already loaded.");
            return;
        }

        // Load the world
        wc = switch (type) {
            case LEVEL -> new WorldCreator(levelPath + worldName, new NamespacedKey(SGM.getInstance(), key));
            case GAME -> new WorldCreator(gamePath + worldName, new NamespacedKey(SGM.getInstance(), key));
            case MAP -> new WorldCreator(mapPath + worldName, new NamespacedKey(SGM.getInstance(), key));
        };
//        WorldCreator wc = new WorldCreator(levelPath + worldName, new NamespacedKey(SGM.getInstance(), key));
        World world = wc.createWorld();
        if (world != null) {
            SGM.getInstance().getCLogger().log("Loaded world: " + worldName);
        } else {
            SGM.getInstance().getCLogger().error("Failed to load world: " + worldName);
        }
    }

    private static void genDataFile(File path, String name, String game) {
        // probably rewriting tmr
        // these are opposite
        Map<String, Object> data = new HashMap<>();
        data.put("name", name);
        data.put("game", game);
        DumperOptions options = new DumperOptions();
        options.setDefaultFlowStyle(DumperOptions.FlowStyle.BLOCK);
        Yaml yaml = new Yaml(options);
        File yamlFile = new File(path + "/sgm.yml");
        try (FileWriter writer = new FileWriter(yamlFile)) {
            yaml.dump(data, writer);
        } catch (IOException e) {
            SGM.getInstance().getCLogger().error(e);
        }
    }

    public static World getLevel(String name) {
        return Bukkit.getWorld(new NamespacedKey(SGM.getInstance(), "level_" + name));
    }

    public LevelResult exportLevel(String name) throws IOException {
        File levelDir = new File(levelPath + name);
        File mapDir = new File(mapPath + name);
        World world = Bukkit.getWorld(new NamespacedKey(SGM.getInstance(), "level_" + name));
        if (world != null) {
            for (Player player : world.getPlayers()) {
                CustomPlayer cPlayer = new CustomPlayer(player);
                cPlayer.sendMessage(SGM.getInstance().getMessages().getString("sgm.level.export.kick"));
                // This is probably gonna be funny later
                player.teleport(Bukkit.getServer().getWorlds().get(0).getSpawnLocation());
            }
            if (!Bukkit.isTickingWorlds() && Bukkit.getServer().unloadWorld(world, true)) {
                FileUtils.copyDirectory(levelDir, mapDir);
                FileUtils.delete(new File(mapDir + "/uid.dat"));
                loadWorld(name, LevelType.LEVEL);
                return LevelResult.SUCCESS;
            }
            return LevelResult.UNKNOWN;
        } else {
            return LevelResult.WORLD_NO_EXISTS;
        }
    }

    public LevelResult createLevel(String name, String game) {
        File level = new File(levelPath + name);
        if (!SGM.getInstance().getGameManager().getRegisteredGames().contains(game.toLowerCase())) {
            return LevelResult.INVALID_ARGS;
        }
        WorldCreator wc = new WorldCreator(levelPath + name, new NamespacedKey(SGM.getInstance(), "level_" + name));
        if (level.exists()) {
            return LevelResult.WORLD_EXISTS;
        }
        wc.type(WorldType.FLAT);
        wc.generatorSettings("{\"layers\": [{\"block\": \"air\", \"height\": 1}], \"biome\":\"plains\"}");
        wc.generateStructures(false);
        World world = wc.createWorld();
        if (world == null) {
            return LevelResult.UNKNOWN;
        }
        Location location = new Location(world, 0.5, 0, 0.5);
        Block block = location.subtract(0, 2, 0).getBlock();
        block.setType(Material.STONE);
        world.setSpawnLocation(location);
        world.setSpawnFlags(false, false);
        genDataFile(level, name, game.toLowerCase());
        enabledMaps.get(game).add(name);
        return LevelResult.SUCCESS;
    }

    public void loadLevels() {
        for (String game : SGM.getInstance().getGameManager().getRegisteredGames()) {
            enabledMaps.put(game, new ArrayList<>()); // Initialize an empty ArrayList for each game
        }
        // List all files (worlds) in the folder
        File folder = new File(mapPath);
        File[] files = folder.listFiles();
        if (files != null) {
            for (File file : files) {
                if (file.isDirectory()) {
                    // Check if sgm.yml exists in the world directory
                    File configFile = new File(file, "sgm.yml");
                    if (configFile.exists()) {
                        YamlConfiguration config = YamlConfiguration.loadConfiguration(configFile);
                        String game = config.getString("game");
                        String name = config.getString("name");
                        if (enabledMaps.containsKey(game)) {
                            enabledMaps.get(game).add(name);
                            // Load the world if sgm.yml exists
                            loadWorld(file.getName(), LevelType.LEVEL);
                        }
                    } else {
                        SGM.getInstance().getCLogger().log("Skipping directory " + file.getName() + ": sgm.yml not found.");
                    }
                }
            }
        }
    }

    public enum LevelResult {
        SUCCESS, WORLD_NO_EXISTS, WORLD_EXISTS, INVALID_ARGS, UNKNOWN
    }

    public enum LevelType {
        GAME, MAP, LEVEL
    }

}